@@ -8,7 +8,7 @@ mod reference;
     any(target_arch = "x86", target_arch = "x86_64"),
     any(target_feature = "sse2", target_feature = "avx2")
 )))]
-pub(crate) type ChaChaEngine = reference::State;
+pub(crate) type ChaChaEngine<const R: usize> = reference::State<R>;
 
 #[cfg(all(
     any(target_arch = "x86", target_arch = "x86_64"),
@@ -20,4 +20,4 @@ mod sse2;
     any(target_arch = "x86", target_arch = "x86_64"),
     target_feature = "sse2",
 ))]
-pub(crate) type ChaChaEngine = sse2::State;
+pub(crate) type ChaChaEngine<const R: usize> = sse2::State<R>;
