@@ -36,22 +36,31 @@ use crate::cryptoutil::{xor_keystream, xor_keystream_mut};
 
 /// ChaCha Context
 #[derive(Clone)]
-pub struct ChaCha20 {
-    state: ChaChaState,
+pub struct ChaCha<const ROUNDS: usize> {
+    state: ChaChaState<ROUNDS>,
     output: [u8; 64],
     offset: usize,
 }
 
-impl ChaCha20 {
+pub type ChaCha20 = ChaCha<20>;
+
+impl ChaCha<20> {
+    pub fn new_xchacha20(key: &[u8], nonce: &[u8]) -> Self {
+        Self::new_xchacha(key, nonce)
+    }
+}
+
+impl<const ROUNDS: usize> ChaCha<ROUNDS> {
     /// Create a new ChaCha20 context.
     ///
     /// * The key must be 16 or 32 bytes
     /// * The nonce must be 8 or 12 bytes
-    pub fn new(key: &[u8], nonce: &[u8]) -> ChaCha20 {
+    pub fn new(key: &[u8], nonce: &[u8]) -> Self {
         assert!(key.len() == 16 || key.len() == 32);
         assert!(nonce.len() == 8 || nonce.len() == 12);
+        assert!(ROUNDS == 8 || ROUNDS == 12 || ROUNDS == 20);
 
-        ChaCha20 {
+        Self {
             state: ChaChaState::init(key, nonce),
             output: [0u8; 64],
             offset: 64,
@@ -61,9 +70,10 @@ impl ChaCha20 {
     /// Create a new XChaCha20 context.
     ///
     /// Key must be 32 bytes and the nonce 24 bytes.
-    pub fn new_xchacha20(key: &[u8], nonce: &[u8]) -> ChaCha20 {
+    pub fn new_xchacha(key: &[u8], nonce: &[u8]) -> Self {
         assert!(key.len() == 32);
         assert!(nonce.len() == 24);
+        assert!(ROUNDS == 8 || ROUNDS == 12 || ROUNDS == 20);
 
         // HChaCha20 produces a 256-bit output block starting from a 512 bit
         // input block where (x0,x1,...,x15) where
@@ -71,27 +81,27 @@ impl ChaCha20 {
         //  * (x0, x1, x2, x3) is the ChaCha20 constant.
         //  * (x4, x5, ... x11) is a 256 bit key.
         //  * (x12, x13, x14, x15) is a 128 bit nonce.
-        let mut xchacha20 = ChaCha20 {
+        let mut xchacha = ChaCha {
             state: ChaChaState::init(key, &nonce[0..16]),
             output: [0u8; 64],
             offset: 64,
         };
 
-        // Use HChaCha to derive the subkey, and initialize a ChaCha20 instance
+        // Use HChaCha to derive the subkey, and initialize a ChaCha<ROUNDS> instance
         // with the subkey and the remaining 8 bytes of the nonce.
         let mut new_key = [0; 32];
-        xchacha20.hchacha20(&mut new_key);
-        xchacha20.state = ChaChaState::init(&new_key, &nonce[16..24]);
+        xchacha.hchacha(&mut new_key);
+        xchacha.state = ChaChaState::init(&new_key, &nonce[16..24]);
 
-        xchacha20
+        xchacha
     }
 
-    fn hchacha20(&mut self, out: &mut [u8; 32]) {
+    fn hchacha(&mut self, out: &mut [u8; 32]) {
         let mut state = self.state.clone();
 
         // Apply r/2 iterations of the same "double-round" function,
         // obtaining (z0, z1, ... z15) = doubleround r/2 (x0, x1, ... x15).
-        state.round20();
+        state.round();
 
         // HChaCha20 then outputs the 256-bit block (z0, z1, z2, z3, z12, z13,
         // z14, z15).  These correspond to the constant and input positions in
@@ -102,7 +112,7 @@ impl ChaCha20 {
     // put the the next 64 keystream bytes into self.output
     fn update(&mut self) {
         let mut state = self.state.clone();
-        state.round20();
+        state.round();
         state.add_back(&self.state);
 
         state.output_bytes(&mut self.output);
@@ -112,7 +122,7 @@ impl ChaCha20 {
     }
 }
 
-impl ChaCha20 {
+impl<const ROUNDS: usize> ChaCha<ROUNDS> {
     /// Process the input in place through the cipher xoring
     pub fn process_mut(&mut self, data: &mut [u8]) {
         let len = data.len();
@@ -162,8 +172,8 @@ impl ChaCha20 {
 
 #[cfg(test)]
 mod test {
-    use std::iter::repeat;
-    use std::vec::Vec;
+    use alloc::vec::Vec;
+    use core::iter::repeat;
 
     use super::ChaCha20;
 
@@ -175,7 +185,7 @@ mod test {
             keystream: &'static [u8],
         }
         // taken from http://tools.ietf.org/html/draft-agl-tls-chacha20poly1305-04
-        let test_vectors = vec![
+        let test_vectors = [
             TestVector {
                 key: [
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -316,10 +326,10 @@ mod test {
         struct TestVector {
             key: [u8; 32],
             nonce: [u8; 12],
-            keystream: Vec<u8>,
+            keystream: &'static [u8],
         }
         // taken from http://tools.ietf.org/html/draft-agl-tls-chacha20poly1305-04
-        let test_vectors = vec![
+        let test_vectors = [
             TestVector {
                 key: [
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -329,7 +339,7 @@ mod test {
                 nonce: [
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                 ],
-                keystream: vec![
+                keystream: &[
                     0x76, 0xb8, 0xe0, 0xad, 0xa0, 0xf1, 0x3d, 0x90, 0x40, 0x5d, 0x6a, 0xe5, 0x53,
                     0x86, 0xbd, 0x28, 0xbd, 0xd2, 0x19, 0xb8, 0xa0, 0x8d, 0xed, 0x1a, 0xa8, 0x36,
                     0xef, 0xcc, 0x8b, 0x77, 0x0d, 0xc7, 0xda, 0x41, 0x59, 0x7c, 0x51, 0x57, 0x48,
@@ -346,7 +356,7 @@ mod test {
                 nonce: [
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                 ],
-                keystream: vec![
+                keystream: &[
                     0x45, 0x40, 0xf0, 0x5a, 0x9f, 0x1f, 0xb2, 0x96, 0xd7, 0x73, 0x6e, 0x7b, 0x20,
                     0x8e, 0x3c, 0x96, 0xeb, 0x4f, 0xe1, 0x83, 0x46, 0x88, 0xd2, 0x60, 0x4f, 0x45,
                     0x09, 0x52, 0xed, 0x43, 0x2d, 0x41, 0xbb, 0xe2, 0xa0, 0xb6, 0xea, 0x75, 0x66,
@@ -363,7 +373,7 @@ mod test {
                 nonce: [
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
                 ],
-                keystream: vec![
+                keystream: &[
                     0xde, 0x9c, 0xba, 0x7b, 0xf3, 0xd6, 0x9e, 0xf5, 0xe7, 0x86, 0xdc, 0x63, 0x97,
                     0x3f, 0x65, 0x3a, 0x0b, 0x49, 0xe0, 0x15, 0xad, 0xbf, 0xf7, 0x13, 0x4f, 0xcb,
                     0x7d, 0xf1, 0x37, 0x82, 0x10, 0x31, 0xe8, 0x5a, 0x05, 0x02, 0x78, 0xa7, 0x08,
@@ -380,7 +390,7 @@ mod test {
                 nonce: [
                     0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                 ],
-                keystream: vec![
+                keystream: &[
                     0xef, 0x3f, 0xdf, 0xd6, 0xc6, 0x15, 0x78, 0xfb, 0xf5, 0xcf, 0x35, 0xbd, 0x3d,
                     0xd3, 0x3b, 0x80, 0x09, 0x63, 0x16, 0x34, 0xd2, 0x1e, 0x42, 0xac, 0x33, 0x96,
                     0x0b, 0xd1, 0x38, 0xe5, 0x0d, 0x32, 0x11, 0x1e, 0x4c, 0xaf, 0x23, 0x7e, 0xe5,
@@ -397,7 +407,7 @@ mod test {
                 nonce: [
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                 ],
-                keystream: vec![
+                keystream: &[
                     0xf7, 0x98, 0xa1, 0x89, 0xf1, 0x95, 0xe6, 0x69, 0x82, 0x10, 0x5f, 0xfb, 0x64,
                     0x0b, 0xb7, 0x75, 0x7f, 0x57, 0x9d, 0xa3, 0x16, 0x02, 0xfc, 0x93, 0xec, 0x01,
                     0xac, 0x56, 0xf8, 0x5a, 0xc3, 0xc1, 0x34, 0xa4, 0x54, 0x7b, 0x73, 0x3b, 0x46,
@@ -435,7 +445,6 @@ mod test {
 #[cfg(all(test, feature = "with-bench"))]
 mod bench {
     use super::ChaCha20;
-    use crate::symmetriccipher::SynchronousStreamCipher;
     use test::Bencher;
 
     #[bench]
