@@ -40,7 +40,7 @@
 //!
 //! let key : [u8; 16] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15];
 //! let nonce : [u8; 8] = [1,2,3,4,5,6,7,8];
-//! let mut context = Context::new(&key, &nonce);
+//! let mut context = Context::<20>::new(&key, &nonce);
 //!
 //! // Add incrementally 2 slices of data
 //! context.add_data(b"authenticated");
@@ -67,11 +67,11 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use crate::chacha20::ChaCha20;
+use crate::chacha20::ChaCha;
+use crate::constant_time::{Choice, CtEqual};
 use crate::cryptoutil::write_u64_le;
 use crate::mac::Mac;
 use crate::poly1305::Poly1305;
-use crate::util::fixed_time_eq;
 
 /// Chacha20Poly1305 Incremental Context for Authenticated Data (AAD)
 ///
@@ -84,7 +84,7 @@ use crate::util::fixed_time_eq;
 ///
 /// let key : [u8; 16] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15];
 /// let nonce : [u8; 8] = [1,2,3,4,5,6,7,8];
-/// let mut context = Context::new(&key, &nonce);
+/// let mut context = Context::<20>::new(&key, &nonce);
 ///
 /// // Add incrementally 2 slices of data
 /// context.add_data(b"authenticated");
@@ -102,8 +102,8 @@ use crate::util::fixed_time_eq;
 /// encrypted_input[10..26].copy_from_slice(&tag.0);
 /// ```
 #[derive(Clone)]
-pub struct Context {
-    cipher: ChaCha20,
+pub struct Context<const ROUNDS: usize> {
+    cipher: ChaCha<ROUNDS>,
     mac: Poly1305,
     aad_len: u64,
     data_len: u64,
@@ -111,25 +111,34 @@ pub struct Context {
 
 /// ChaCha20Poly1305 Incremental Context for encryption
 #[derive(Clone)]
-pub struct ContextEncryption(Context);
+pub struct ContextEncryption<const ROUNDS: usize>(Context<ROUNDS>);
 
 /// ChaCha20Poly1305 Incremental Context for decryption
 #[derive(Clone)]
-pub struct ContextDecryption(Context);
+pub struct ContextDecryption<const ROUNDS: usize>(Context<ROUNDS>);
 
 /// ChaCha20Poly1305 Authenticated Tag (128 bits)
 #[derive(Debug, Clone)]
 pub struct Tag(pub [u8; 16]);
 
+impl CtEqual for &Tag {
+    fn ct_eq(self, other: Self) -> Choice {
+        self.0.ct_eq(&other.0)
+    }
+    fn ct_ne(self, b: Self) -> Choice {
+        self.ct_eq(b).negate()
+    }
+}
+
 impl PartialEq for Tag {
     fn eq(&self, other: &Self) -> bool {
-        fixed_time_eq(&self.0, &other.0)
+        self.ct_eq(other).is_true()
     }
 }
 
 impl Eq for Tag {}
 
-impl Context {
+impl<const ROUNDS: usize> Context<ROUNDS> {
     /// Create a new context given the key and nonce.
     ///
     /// ```
@@ -137,12 +146,12 @@ impl Context {
     ///
     /// let key : [u8; 16] = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15];
     /// let nonce : [u8; 8] = [1,2,3,4,5,6,7,8];
-    /// let context = Context::new(&key, &nonce);
+    /// let context = Context::<20>::new(&key, &nonce);
     /// ```
     pub fn new(key: &[u8], nonce: &[u8]) -> Self {
         assert!(key.len() == 16 || key.len() == 32);
         assert!(nonce.len() == 8 || nonce.len() == 12);
-        let mut cipher = ChaCha20::new(key, nonce);
+        let mut cipher = ChaCha::new(key, nonce);
         let mut mac_key = [0u8; 64];
         let zero_key = [0u8; 64];
         cipher.process(&zero_key, &mut mac_key);
@@ -170,19 +179,19 @@ impl Context {
     }
 
     /// Finish authenticated part and move to the encryption phase
-    pub fn to_encryption(mut self) -> ContextEncryption {
+    pub fn to_encryption(mut self) -> ContextEncryption<ROUNDS> {
         pad16(&mut self.mac, self.aad_len);
         ContextEncryption(self)
     }
 
     /// Finish authenticated part and move to the decryption phase
-    pub fn to_decryption(mut self) -> ContextDecryption {
+    pub fn to_decryption(mut self) -> ContextDecryption<ROUNDS> {
         pad16(&mut self.mac, self.aad_len);
         ContextDecryption(self)
     }
 }
 
-fn finalize_raw(inner: &mut Context) -> [u8; 16] {
+fn finalize_raw<const ROUNDS: usize>(inner: &mut Context<ROUNDS>) -> [u8; 16] {
     let mut len_buf = [0u8; 16];
     pad16(&mut inner.mac, inner.data_len);
     write_u64_le(&mut len_buf[0..8], inner.aad_len);
@@ -192,7 +201,7 @@ fn finalize_raw(inner: &mut Context) -> [u8; 16] {
     len_buf
 }
 
-impl ContextEncryption {
+impl<const ROUNDS: usize> ContextEncryption<ROUNDS> {
     /// Encrypt input in place
     pub fn encrypt_mut(&mut self, buf: &mut [u8]) {
         self.0.cipher.process_mut(buf);
@@ -227,7 +236,7 @@ pub enum DecryptionResult {
     MisMatch,
 }
 
-impl ContextDecryption {
+impl<const ROUNDS: usize> ContextDecryption<ROUNDS> {
     /// Decrypt input in place
     pub fn decrypt_mut(&mut self, buf: &mut [u8]) {
         self.0.add_encrypted(buf);
@@ -259,9 +268,9 @@ impl ContextDecryption {
 
 /// A ChaCha20+Poly1305 Context
 #[derive(Clone)]
-pub struct ChaCha20Poly1305 {
+pub struct ChaChaPoly1305<const ROUNDS: usize> {
     finished: bool,
-    context: Context,
+    context: Context<ROUNDS>,
 }
 
 fn pad16(mac: &mut Poly1305, len: u64) {
@@ -272,16 +281,18 @@ fn pad16(mac: &mut Poly1305, len: u64) {
     }
 }
 
-impl ChaCha20Poly1305 {
+pub type ChaCha20Poly1305 = ChaChaPoly1305<20>;
+
+impl<const ROUNDS: usize> ChaChaPoly1305<ROUNDS> {
     /// Create a new ChaCha20Poly1305
     ///
     /// * key needs to be 16 or 32 bytes
     /// * nonce needs to be 8 or 12 bytes
     ///
-    pub fn new(key: &[u8], nonce: &[u8], aad: &[u8]) -> ChaCha20Poly1305 {
+    pub fn new(key: &[u8], nonce: &[u8], aad: &[u8]) -> Self {
         let mut context = Context::new(key, nonce);
         context.add_data(aad);
-        ChaCha20Poly1305 {
+        ChaChaPoly1305 {
             context: context,
             finished: false,
         }
@@ -378,15 +389,14 @@ impl ChaCha20Poly1305 {
 #[cfg(test)]
 mod test {
     use super::ChaCha20Poly1305;
-    use std::vec::Vec;
 
     struct TestVector {
         key: [u8; 32],
         nonce: &'static [u8],
         tag: [u8; 16],
-        plain_text: Vec<u8>,
-        cipher_text: Vec<u8>,
-        aad: Vec<u8>,
+        plain_text: &'static [u8],
+        cipher_text: &'static [u8],
+        aad: &'static [u8],
     }
 
     fn test_vector(v: &TestVector) {
@@ -422,7 +432,7 @@ mod test {
                 nonce: &[
                     0x07, 0x00, 0x00, 0x00, 0x40, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47,
                 ],
-                plain_text: vec![
+                plain_text: &[
                     0x4c, 0x61, 0x64, 0x69, 0x65, 0x73, 0x20, 0x61, 0x6e, 0x64, 0x20, 0x47, 0x65,
                     0x6e, 0x74, 0x6c, 0x65, 0x6d, 0x65, 0x6e, 0x20, 0x6f, 0x66, 0x20, 0x74, 0x68,
                     0x65, 0x20, 0x63, 0x6c, 0x61, 0x73, 0x73, 0x20, 0x6f, 0x66, 0x20, 0x27, 0x39,
@@ -433,10 +443,10 @@ mod test {
                     0x20, 0x73, 0x75, 0x6e, 0x73, 0x63, 0x72, 0x65, 0x65, 0x6e, 0x20, 0x77, 0x6f,
                     0x75, 0x6c, 0x64, 0x20, 0x62, 0x65, 0x20, 0x69, 0x74, 0x2e,
                 ],
-                aad: vec![
+                aad: &[
                     0x50, 0x51, 0x52, 0x53, 0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7,
                 ],
-                cipher_text: vec![
+                cipher_text: &[
                     0xd3, 0x1a, 0x8d, 0x34, 0x64, 0x8e, 0x60, 0xdb, 0x7b, 0x86, 0xaf, 0xbc, 0x53,
                     0xef, 0x7e, 0xc2, 0xa4, 0xad, 0xed, 0x51, 0x29, 0x6e, 0x08, 0xfe, 0xa9, 0xe2,
                     0xb5, 0xa7, 0x36, 0xee, 0x62, 0xd6, 0x3d, 0xbe, 0xa4, 0x5e, 0x8c, 0xa9, 0x67,
@@ -466,7 +476,7 @@ mod test {
                     0xee, 0xad, 0x9d, 0x67, 0x89, 0x0c, 0xbb, 0x22, 0x39, 0x23, 0x36, 0xfe, 0xa1,
                     0x85, 0x1f, 0x38,
                 ],
-                plain_text: vec![
+                plain_text: &[
                     0x49, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x65, 0x74, 0x2d, 0x44, 0x72, 0x61, 0x66,
                     0x74, 0x73, 0x20, 0x61, 0x72, 0x65, 0x20, 0x64, 0x72, 0x61, 0x66, 0x74, 0x20,
                     0x64, 0x6f, 0x63, 0x75, 0x6d, 0x65, 0x6e, 0x74, 0x73, 0x20, 0x76, 0x61, 0x6c,
@@ -489,7 +499,7 @@ mod test {
                     0x6b, 0x20, 0x69, 0x6e, 0x20, 0x70, 0x72, 0x6f, 0x67, 0x72, 0x65, 0x73, 0x73,
                     0x2e, 0x2f, 0xe2, 0x80, 0x9d,
                 ],
-                cipher_text: vec![
+                cipher_text: &[
                     0x64, 0xa0, 0x86, 0x15, 0x75, 0x86, 0x1a, 0xf4, 0x60, 0xf0, 0x62, 0xc7, 0x9b,
                     0xe6, 0x43, 0xbd, 0x5e, 0x80, 0x5c, 0xfd, 0x34, 0x5c, 0xf3, 0x89, 0xf1, 0x08,
                     0x67, 0x0a, 0xc7, 0x6c, 0x8c, 0xb2, 0x4c, 0x6c, 0xfc, 0x18, 0x75, 0x5d, 0x43,
@@ -512,7 +522,7 @@ mod test {
                     0xdf, 0x99, 0x7b, 0x71, 0x4d, 0x6c, 0x6f, 0x2c, 0x29, 0xa6, 0xad, 0x5c, 0xb4,
                     0x02, 0x2b, 0x02, 0x70, 0x9b,
                 ],
-                aad: vec![
+                aad: &[
                     0xf3, 0x33, 0x88, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x4e, 0x91,
                 ],
             },
