@@ -20,7 +20,6 @@ use alloc::vec::Vec;
 use core::iter::repeat;
 use core::mem::size_of;
 
-use crate::cryptoutil::copy_memory;
 use crate::cryptoutil::{read_u32_le, read_u32v_le, write_u32_le};
 use crate::hmac::Hmac;
 use crate::pbkdf2::pbkdf2;
@@ -95,7 +94,12 @@ fn xor(x: &[u8], y: &[u8], output: &mut [u8]) {
 // output - the output vector. Must be the same length as input.
 fn scrypt_block_mix(input: &[u8], output: &mut [u8]) {
     let mut x = [0u8; 64];
-    copy_memory(&input[input.len() - 64..], &mut x);
+    let left_over = input.len() % 64;
+    if left_over > 0 {
+        x[0..left_over].copy_from_slice(&input[input.len() - 64..]);
+    } else {
+        x.copy_from_slice(&input[input.len() - 64..]);
+    }
 
     let mut t = [0u8; 64];
 
@@ -107,7 +111,7 @@ fn scrypt_block_mix(input: &[u8], output: &mut [u8]) {
         } else {
             (i / 2) * 64 + input.len() / 2
         };
-        copy_memory(&x, &mut output[pos..pos + 64]);
+        output[pos..pos + 64].copy_from_slice(&x);
     }
 }
 
@@ -130,7 +134,7 @@ fn scrypt_ro_mix(b: &mut [u8], v: &mut [u8], t: &mut [u8], n: usize) {
     let len = b.len();
 
     for chunk in v.chunks_mut(len) {
-        copy_memory(b, chunk);
+        chunk[0..b.len()].copy_from_slice(b);
         scrypt_block_mix(chunk, b);
     }
 
@@ -265,20 +269,20 @@ mod test {
         log_n: u8,
         r: u32,
         p: u32,
-        expected: Vec<u8>,
+        expected: &'static [u8],
     }
 
     // Test vectors from [1]. The last test vector is omitted because it takes too long to run.
 
-    fn tests() -> Vec<Test> {
-        vec![
+    fn tests() -> &'static [Test] {
+        &[
             Test {
                 password: "",
                 salt: "",
                 log_n: 4,
                 r: 1,
                 p: 1,
-                expected: vec![
+                expected: &[
                     0x77, 0xd6, 0x57, 0x62, 0x38, 0x65, 0x7b, 0x20, 0x3b, 0x19, 0xca, 0x42, 0xc1,
                     0x8a, 0x04, 0x97, 0xf1, 0x6b, 0x48, 0x44, 0xe3, 0x07, 0x4a, 0xe8, 0xdf, 0xdf,
                     0xfa, 0x3f, 0xed, 0xe2, 0x14, 0x42, 0xfc, 0xd0, 0x06, 0x9d, 0xed, 0x09, 0x48,
@@ -292,7 +296,7 @@ mod test {
                 log_n: 10,
                 r: 8,
                 p: 16,
-                expected: vec![
+                expected: &[
                     0xfd, 0xba, 0xbe, 0x1c, 0x9d, 0x34, 0x72, 0x00, 0x78, 0x56, 0xe7, 0x19, 0x0d,
                     0x01, 0xe9, 0xfe, 0x7c, 0x6a, 0xd7, 0xcb, 0xc8, 0x23, 0x78, 0x30, 0xe7, 0x73,
                     0x76, 0x63, 0x4b, 0x37, 0x31, 0x62, 0x2e, 0xaf, 0x30, 0xd9, 0x2e, 0x22, 0xa3,
@@ -306,7 +310,7 @@ mod test {
                 log_n: 14,
                 r: 8,
                 p: 1,
-                expected: vec![
+                expected: &[
                     0x70, 0x23, 0xbd, 0xcb, 0x3a, 0xfd, 0x73, 0x48, 0x46, 0x1c, 0x06, 0xcd, 0x81,
                     0xfd, 0x38, 0xeb, 0xfd, 0xa8, 0xfb, 0xba, 0x90, 0x4f, 0x8e, 0x3e, 0xa9, 0xb5,
                     0x43, 0xf6, 0x54, 0x5d, 0xa1, 0xf2, 0xd5, 0x43, 0x29, 0x55, 0x61, 0x3f, 0x0f,
@@ -329,7 +333,7 @@ mod test {
                 &params,
                 &mut result,
             );
-            assert!(result == t.expected);
+            assert_eq!(result, t.expected);
         }
     }
 }
